"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const mp3_duration_1 = tslib_1.__importDefault(require("mp3-duration"));
const log_1 = require("../utils/log");
const defaultPaths_1 = require("../config/defaultPaths");
class RetrieveAudioDataService {
    constructor(content) {
        this.transitionDurationInSeconds = 2.9;
        this.content = content;
    }
    async execute(audioDataIsInContent, options) {
        if (!audioDataIsInContent) {
            await this.retrieveAudioData(options.haveIntro || false, options.haveEnd || false);
        }
        if (!this.content.renderData) {
            log_1.error('RenderData is undefined', 'RetrieveAudioDataService');
            return;
        }
        for (let i = 0; i < this.content.renderData.length; i++) {
            log_1.log(`Getting duration of audio-sentence ${path_1.default.basename(this.content.renderData[i].audioFilePath)}`, 'RetrieveAudioDataService');
            const duration = await this.getDuration(this.content.renderData[i].audioFilePath);
            this.content.renderData[i].duration = duration;
        }
        this.setFullDuration();
    }
    async retrieveAudioData(haveIntro, haveEnd) {
        var _a, _b;
        log_1.log('Getting data from audio files in tmp/', 'RetrieveAudioDataService');
        const tmpPath = await defaultPaths_1.getPath('tmp');
        const files = fs_1.default.readdirSync(tmpPath);
        const introFilePath = 'output-intro.mp3';
        const endFilePath = 'output-end.mp3';
        const introAndEndFilesRegExp = new RegExp('-intro.*|-end.*');
        const sortedNewsFiles = files
            .filter(file => path_1.default.extname(file) === '.mp3' &&
            !file.match(introAndEndFilesRegExp))
            .map(file => Number(file.split('-')[1].split('.')[0]))
            .sort((a, b) => a - b)
            .map(fileNumber => {
            const fileRegExp = new RegExp(`-${fileNumber}.mp3`);
            return files.find(file => file.match(fileRegExp));
        });
        if (typeof sortedNewsFiles === 'undefined' ||
            sortedNewsFiles.length <= 0) {
            log_1.error("Couldn't not find audio files", 'RetrieveAudioDataService');
        }
        if (typeof this.content.renderData === 'undefined') {
            this.content.renderData = [];
        }
        if (haveIntro &&
            fs_1.default.existsSync(path_1.default.resolve(tmpPath, introFilePath)) &&
            ((_a = this.content.intro) === null || _a === void 0 ? void 0 : _a.text)) {
            this.content.renderData.push({
                text: this.content.intro.text,
                duration: 0,
                audioFilePath: path_1.default.resolve(tmpPath, introFilePath),
            });
        }
        for (let i = 0; i < this.content.news.length; i++) {
            this.content.renderData.push({
                text: this.content.news[i].text,
                duration: 0,
                audioFilePath: path_1.default.resolve(tmpPath, sortedNewsFiles[i]),
            });
        }
        if (haveEnd &&
            fs_1.default.existsSync(path_1.default.resolve(tmpPath, endFilePath)) &&
            ((_b = this.content.end) === null || _b === void 0 ? void 0 : _b.text)) {
            this.content.renderData.push({
                text: this.content.end.text,
                duration: 0,
                audioFilePath: path_1.default.resolve(tmpPath, endFilePath),
            });
        }
        console.log(this.content);
    }
    async getDuration(filePath) {
        return new Promise(resolve => {
            mp3_duration_1.default(filePath, (err, duration) => {
                if (err) {
                    log_1.error(`Failed at getting duration of ${filePath}`, `RetrieveAudioDataService`);
                }
                resolve(duration);
            });
        });
    }
    setFullDuration() {
        if (!this.content.renderData) {
            log_1.error('RenderData is undefined', 'RetrieveAudioDataService');
            return;
        }
        this.content.fullDuration = this.content.renderData.reduce((accumulator, currentValue, index) => {
            if (!this.content.renderData ||
                index !== this.content.renderData.length - 1) {
                return (accumulator +
                    currentValue.duration +
                    this.transitionDurationInSeconds);
            }
            return accumulator + currentValue.duration;
        }, 0);
    }
}
exports.default = RetrieveAudioDataService;
